from typing import Dict
from sddl_parser.rights_enums import (
    GenericAccessRights,
    FileAccessRights,
    RegistryKeyAccessRights,
)
from sddl_parser.type_enums import AceType

# https://learn.microsoft.com/en-us/openspecs/windows_protocols/ms-dtyp/628ebb1d-c509-4ea0-a10f-77ef97ca4586
ACE_TYPE: Dict[str, AceType] = {
    "A": AceType.ACCESS_ALLOWED,
    "D": AceType.ACCESS_DENIED,
    "OA": AceType.ACCESS_ALLOWED_OBJECT,
    "OD": AceType.ACCESS_DENIED_OBJECT,
    "AU": AceType.SYSTEM_AUDIT,
    "AL": AceType.SYSTEM_ALARM,
    "OU": AceType.SYSTEM_AUDIT_OBJECT,
    "OL": AceType.SYSTEM_ALARM_OBJECT,
    "ML": AceType.SYSTEM_MANDATORY_LABEL,
    "XA": AceType.ACCESS_ALLOWED_CALLBACK,
    "XD": AceType.ACCESS_DENIED_CALLBACK,
    "RA": AceType.SYSTEM_RESOURCE_ATTRIBUTE,
    "SP": AceType.SYSTEM_SCOPED_POLICY_ID,
    "XU": AceType.SYSTEM_AUDIT_CALLBACK,
    "ZA": AceType.ACCESS_ALLOWED_CALLBACK_OBJECT,
    "TL": AceType.SYSTEM_PROCESS_TRUST_LABEL,
    "FL": AceType.SYSTEM_ACCESS_FILTER,
}

ACE_FLAGS = {
    "CI": ("CONTAINER_INHERIT", "CONTAINER_INHERIT_ACE", 0x02),
    "OI": ("OBJECT_INHERIT", "OBJECT_INHERIT_ACE", 0x01),
    "NP": ("NO_PROPAGATE", "NO_PROPAGATE_INHERIT_ACE", 0x04),
    "IO": ("INHERIT_ONLY", "INHERIT_ONLY_ACE", 0x08),
    "ID": ("INHERITED", "INHERITED_ACE", 0x10),
    "SA": ("AUDIT_SUCCESS", "SUCCESSFUL_ACCESS_ACE_FLAG", 0x40),
    "FA": ("AUDIT_FAILURE", "FAILED_ACCESS_ACE_FLAG", 0x80),
    # TODO: TP and CR missing - https://learn.microsoft.com/en-us/windows/win32/secauthz/ace-strings
}


# https://github.com/tpn/winsdk-10/blob/master/Include/10.0.14393.0/shared/sddl.h#L123
ACE_RIGHTS: Dict[str, int] = {
    "CC": GenericAccessRights.ACCESS0,
    "DC": GenericAccessRights.ACCESS1,
    "LC": GenericAccessRights.ACCESS2,
    "SW": GenericAccessRights.ACCESS3,
    "RP": GenericAccessRights.ACCESS4,
    "WP": GenericAccessRights.ACCESS5,
    "DT": GenericAccessRights.ACCESS6,
    "LO": GenericAccessRights.ACCESS7,
    "CR": GenericAccessRights.ACCESS8,
    "SD": GenericAccessRights.DELETE,
    "RC": GenericAccessRights.READ_CONTROL,
    "WD": GenericAccessRights.WRITE_DAC,
    "WO": GenericAccessRights.WRITE_OWNER,
    "GA": GenericAccessRights.GENERIC_ALL,
    "GX": GenericAccessRights.GENERIC_EXECUTE,
    "GW": GenericAccessRights.GENERIC_WRITE,
    "GR": GenericAccessRights.GENERIC_READ,
    "FA": FileAccessRights.FILE_ALL_ACCESS,
    "FR": FileAccessRights.FILE_GENERIC_READ,
    "FW": FileAccessRights.FILE_GENERIC_WRITE,
    "FX": FileAccessRights.FILE_GENERIC_EXECUTE,
    "KA": RegistryKeyAccessRights.KEY_ALL_ACCESS,
    "KR": RegistryKeyAccessRights.KEY_READ,
    "KW": RegistryKeyAccessRights.KEY_WRITE,
    "KX": RegistryKeyAccessRights.KEY_EXECUTE,
    # SDDL_NO_READ_UP
    "NR": 0x01,
    # SDDL_NO_WRITE_UP
    "NW": 0x02,
    # SDDL_NO_EXECUTE_UP
    "NX": 0x04,
}

SDDL_SIDS = {  # Well known SIDs
    "AA": "ACCESS_CONTROL_ASSISTANCE_OPS",
    "AC": "ALL_APP_PACKAGES",
    "AN": "ANONYMOUS",
    "AO": "ACCOUNT_OPERATORS",
    "AP": "PROTECTED_USERS",
    "AU": "AUTHENTICATED_USERS",
    "BA": "BUILTIN_ADMINISTRATORS",
    "BG": "BUILTIN_GUESTS",
    "BO": "BACKUP_OPERATORS",
    "BU": "BUILTIN_USERS",
    "CA": "CERT_SERV_ADMINISTRATORS",
    "CD": "CERTSVC_DCOM_ACCESS",
    "CG": "CREATOR_GROUP",
    "CN": "CLONEABLE_CONTROLLERS",
    "CO": "CREATOR_OWNER",
    "CY": "CRYPTO_OPERATORS",
    "DA": "DOMAIN_ADMINISTRATORS",
    "DC": "DOMAIN_COMPUTERS",
    "DD": "DOMAIN_DOMAIN_CONTROLLERS",
    "DG": "DOMAIN_GUESTS",
    "DU": "DOMAIN_USERS",
    "EA": "ENTERPRISE_ADMINS",
    "ED": "ENTERPRISE_DOMAIN_CONTROLLERS",
    "EK": "ENTERPRISE_KEY_ADMINS",
    "ER": "EVENT_LOG_READERS",
    "ES": "RDS_ENDPOINT_SERVERS",
    "HA": "HYPER_V_ADMINS",
    "HI": "ML_HIGH",
    "IS": "IIS_USERS",
    "IU": "INTERACTIVE",
    "KA": "KEY_ADMINS",
    "LA": "LOCAL_ADMIN",
    "LG": "LOCAL_GUEST",
    "LS": "LOCAL_SERVICE",
    "LU": "PERFLOG_USERS",
    "LW": "ML_LOW",
    "ME": "ML_MEDIUM",
    "MP": "ML_MEDIUM_PLUS",
    "MU": "PERFMON_USERS",
    "NO": "NETWORK_CONFIGURATION_OPS",
    "NS": "NETWORK_SERVICE",
    "NU": "NETWORK",
    "OW": "OWNER_RIGHTS",
    "PA": "GROUP_POLICY_ADMINS",
    "PO": "PRINTER_OPERATORS",
    "PS": "PERSONAL_SELF",
    "PU": "POWER_USERS",
    "RA": "RDS_REMOTE_ACCESS_SERVERS",
    "RC": "RESTRICTED_CODE",
    "RD": "REMOTE_DESKTOP",
    "RE": "REPLICATOR",
    "RM": "RMS__SERVICE_OPERATORS",
    "RO": "ENTERPRISE_RO_DCs",
    "RS": "RAS_SERVERS",
    "RU": "ALIAS_PREW2KCOMPACC",
    "SA": "SCHEMA_ADMINISTRATORS",
    "SI": "ML_SYSTEM",
    "SO": "SERVER_OPERATORS",
    "SS": "SERVICE_ASSERTED",
    "SU": "SERVICE",
    "SY": "LOCAL_SYSTEM",
    "UD": "USER_MODE_DRIVERS",
    "WD": "EVERYONE",
    "WR": "WRITE_RESTRICTED_CODE",
}

SDDL_FLAGS = {
    # Order matters because of the way the flags are checked
    "PAI": "PROTECTED|SDDL_AUTO_INHERITED",
    "AR": "SDDL_AUTO_INHERIT_REQ",
    "AI": "SDDL_AUTO_INHERITED",
    "P": "PROTECTED",
    "": "NULL_DACL",
}
